@@ -5,12 +5,12 @@ require 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '3.57';
+our $VERSION = '3.59';
 $VERSION = eval $VERSION;
 
-use Object::InsideOut::Exception 3.57;
-use Object::InsideOut::Util 3.57 qw(create_object hash_re is_it make_shared);
-use Object::InsideOut::Metadata 3.57;
+use Object::InsideOut::Exception 3.59;
+use Object::InsideOut::Util 3.59 qw(create_object hash_re is_it make_shared);
+use Object::InsideOut::Metadata 3.59;
 
 require B;
 
@@ -1166,7 +1166,7 @@ sub CLONE
     }
 
     # Process thread-shared objects
-    {
+    if (($] < 5.010) || ($threads::shared::VERSION lt '1.15')) {
         my $sh_obj = $GBL{'share'}{'obj'};
         lock($sh_obj) if $GBL{'share'}{'ok'};
 
@@ -1279,12 +1279,14 @@ sub _obj :Sub(Private)
         threads::shared::share($self);
 
         # Add thread tracking list for this thread-shared object
-        my $sh_obj = $GBL{'share'}{'obj'};
-        lock($sh_obj) if $GBL{'share'}{'ok'};
-        if (exists($$sh_obj{$class})) {
-            $$sh_obj{$class}{$$self} = make_shared([ $GBL{'tid'} ]);
-        } else {
-            $$sh_obj{$class} = make_shared({ $$self => [ $GBL{'tid'} ] });
+        if (($] < 5.010) || ($threads::shared::VERSION lt '1.15')) {
+            my $sh_obj = $GBL{'share'}{'obj'};
+            lock($sh_obj) if $GBL{'share'}{'ok'};
+            if (exists($$sh_obj{$class})) {
+                $$sh_obj{$class}{$$self} = make_shared([ $GBL{'tid'} ]);
+            } else {
+                $$sh_obj{$class} = make_shared({ $$self => [ $GBL{'tid'} ] });
+            }
         }
 
     } elsif ($threads::threads) {
@@ -1469,7 +1471,7 @@ sub _args :Sub(Private)
             # Otherwise, check for a specific class or ref type
             # Exact spelling and case required
             else {
-                if ($type =~ /^(array|hash)(?:_?ref)?\s*(?:\(\s*(\S+)\s*\))*$/i) {
+                if ($type =~ /^(array|hash|scalar)(?:_?ref)?\s*(?:\(\s*(\S+)\s*\))*$/i) {
                     $type = uc($1);
                     if (defined($2)) {
                         $subtype = $2;
@@ -1830,7 +1832,7 @@ sub DESTROY
             if ($GBL{'term'}) {
                 return if ($tid);   # Continue only if main thread
 
-            } else {
+            } elsif (($] < 5.010) || ($threads::shared::VERSION lt '1.15')) {
                 my $so_cl = $GBL{'share'}{'obj'}{$class};
                 if (! exists($$so_cl{$$self})) {
                     warn("ERROR: Attempt to DESTROY object ID $$self of class $class in thread ID $tid twice\n");
@@ -1838,8 +1840,12 @@ sub DESTROY
                 }
 
                 # Remove thread ID from this object's thread tracking list
+                # NOTE:  The threads->object() test was added for the case
+                # where OIO objects are passed via Thead::Queue.  I don't
+                # know if this will cause problems with detached threads as
+                # threads->object() returns undef for them.
                 lock($so_cl);
-                if (@{$$so_cl{$$self}} = grep { $_ != $tid } @{$$so_cl{$$self}}) {
+                if (@{$$so_cl{$$self}} = grep { $_ != $tid && threads->object($_) } @{$$so_cl{$$self}}) {
                     return;
                 }
 
@@ -2179,7 +2185,7 @@ sub create_accessors :Sub(Private)
                         $val = 'list';
                     } elsif (uc($val) eq 'HASH') {
                         $val = 'HASH';
-                    } elsif ($val =~ /^(hash|array)_?ref$/i) {
+                    } elsif ($val =~ /^(hash|array|scalar)_?ref$/i) {
                         $val = uc($1) . '_ref';
                     }
                     if ($subtype) {
@@ -2370,7 +2376,7 @@ sub create_accessors :Sub(Private)
     }
 
     # Get type checking (if any)
-    my ($type, $subtype, $ah_ref) = ('NONE', '', 0);
+    my ($type, $subtype, $is_ref) = ('NONE', '', 0);
     if ($$fld_type{$field_ref}) {
         $type = $$fld_type{$field_ref}{'type'};
         if (! ref($type)) {
@@ -2378,9 +2384,9 @@ sub create_accessors :Sub(Private)
                 $type = $1;
                 $subtype = $2;
             }
-            if ($type =~ /^(HASH|ARRAY)_ref$/) {
+            if ($type =~ /^(HASH|ARRAY|SCALAR)_ref$/) {
                 $type = $1;
-                $ah_ref = 1;
+                $is_ref = 1;
             }
         }
     }
@@ -2431,7 +2437,7 @@ sub create_accessors :Sub(Private)
     # Create an :lvalue accessor
     if ($lvalue) {
         $code .= create_lvalue_accessor($pkg, $set, $field_ref, $get,
-                                        $type, $ah_ref, $subtype,
+                                        $type, $is_ref, $subtype,
                                         $name, $return, $private,
                                         $restricted, $weak, $pre);
     }
@@ -2483,7 +2489,7 @@ _PRE_
 
         # Add data type checking
         my ($type_code, $arg_str) = type_code($pkg, $set, $weak,
-                                              $type, $ah_ref, $subtype);
+                                              $type, $is_ref, $subtype);
         $code .= $type_code;
 
         # Add field locking code if sharing
@@ -2588,7 +2594,7 @@ _RESTRICTED_
 # Generate type checking code
 sub type_code :Sub(Private)
 {
-    my ($pkg, $name, $weak, $type, $ah_ref, $subtype) = @_;
+    my ($pkg, $name, $weak, $type, $is_ref, $subtype) = @_;
     my $code = '';
     my $arg_str = '$_[1]';
 
@@ -2662,7 +2668,7 @@ _NUMERIC_
 _ARRAY_
         $arg_str = '$arg';
 
-    } elsif ($type eq 'HASH' && !$ah_ref) {
+    } elsif ($type eq 'HASH' && !$is_ref) {
         # Hash - pairs of args or hash ref
         $code = <<"_HASH_";
     my \$arg;
